#define CDBISDN_CDB_FILE		"/usr/share/hwinfo/ISDN.CDB.txt"
#define CDBISDN_HWDB_FILE		"/usr/share/hwinfo/ISDN.CDB.hwdb"

#define IWHREC_TYPE_VERSION		1
#define IWHREC_TYPE_DATE		2
#define IWHREC_TYPE_NAME_SIZE		3
#define IWHREC_TYPE_NAME_COUNT		4
#define	IWHREC_TYPE_NAME_DATA		5
#define IWHREC_TYPE_VENDOR_COUNT	11
#define IWHREC_TYPE_VENDOR_RECORD	12
#define IWHREC_TYPE_CARD_COUNT		21
#define IWHREC_TYPE_CARD_RECORD		22
#define IWHREC_TYPE_CARD_IDSORTED	23
#define IWHREC_TYPE_VARIO_COUNT		31
#define IWHREC_TYPE_VARIO_RECORD	32

